"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AggregateResponseType = void 0;
const tslib_1 = require("tslib");
const graphql_1 = require("@nestjs/graphql");
const nestjs_query_core_1 = require("@rezonate/nestjs-query-core");
const common_1 = require("../../common");
const decorators_1 = require("../../decorators");
const filterable_field_decorator_1 = require("../../decorators/filterable-field.decorator");
const reflector = new nestjs_query_core_1.MapReflector('nestjs-query:aggregate-response-type');
const capitalize = (key) => key.slice(0, 1).toUpperCase() + key.slice(1);
function NumberAggregatedType(name, fields, NumberType, relationFields = new Map()) {
    const fieldNames = fields.map((f) => f.propertyName);
    let Aggregated = class Aggregated {
    };
    Aggregated = tslib_1.__decorate([
        (0, graphql_1.ObjectType)(name)
    ], Aggregated);
    fieldNames.forEach((propertyName) => {
        (0, graphql_1.Field)(() => NumberType, { nullable: true })(Aggregated.prototype, propertyName);
    });
    relationFields.forEach((related, key) => {
        const rt = NumberAggregatedType(capitalize(key) + name, related, NumberType);
        (0, graphql_1.Field)(() => rt, { nullable: true })(Aggregated.prototype, key);
    });
    return Aggregated;
}
function AggregateGroupByType(name, fields, relationFields = new Map()) {
    let Aggregated = class Aggregated {
    };
    Aggregated = tslib_1.__decorate([
        (0, graphql_1.ObjectType)(name)
    ], Aggregated);
    fields.forEach(({ propertyName, target, returnTypeFunc }) => {
        const rt = returnTypeFunc ? returnTypeFunc() : target;
        (0, graphql_1.Field)(() => rt, { nullable: true })(Aggregated.prototype, propertyName);
    });
    relationFields.forEach((related, key) => {
        const rt = AggregateGroupByType(capitalize(key) + name, related);
        (0, graphql_1.Field)(() => rt, { nullable: true })(Aggregated.prototype, key);
    });
    return Aggregated;
}
function AggregatedType(name, fields, relationFields = new Map()) {
    let Aggregated = class Aggregated {
    };
    Aggregated = tslib_1.__decorate([
        (0, graphql_1.ObjectType)(name)
    ], Aggregated);
    fields.forEach(({ propertyName, target, returnTypeFunc }) => {
        const rt = returnTypeFunc ? returnTypeFunc() : target;
        (0, graphql_1.Field)(() => rt, { nullable: true })(Aggregated.prototype, propertyName);
    });
    relationFields.forEach((related, key) => {
        const rt = AggregateGroupByType(capitalize(key) + name, related);
        (0, graphql_1.Field)(() => rt, { nullable: true })(Aggregated.prototype, key);
    });
    return Aggregated;
}
function AggregateResponseType(DTOClass, opts) {
    const objName = (0, common_1.getGraphqlObjectName)(DTOClass, 'Unable to make AggregationResponseType.');
    const prefix = opts?.prefix ?? objName;
    const aggName = `${prefix}AggregateResponse`;
    return reflector.memoize(DTOClass, aggName, () => {
        const fields = (0, decorators_1.getFilterableFields)(DTOClass);
        const relationFields = (0, filterable_field_decorator_1.getFilterableRelationFields)(DTOClass, ['one', 'many']);
        if (!fields.length && !relationFields.length) {
            throw new Error(`No fields found to create AggregationResponseType for ${DTOClass.name}. Ensure fields are annotated with @FilterableField`);
        }
        const groupedRelationFields = relationFields.reduce((map, field) => {
            map.set(field.relationPropertyName, [...(map.get(field.relationPropertyName) ?? []), field]);
            return map;
        }, new Map());
        const numberFields = fields.filter(({ target }) => target === Number);
        const minMaxFields = fields.filter(({ target }) => target !== Boolean);
        const GroupType = AggregateGroupByType(`${prefix}AggregateGroupBy`, fields, groupedRelationFields);
        const CountType = NumberAggregatedType(`${prefix}CountAggregate`, fields, graphql_1.Int, groupedRelationFields);
        const DistinctCountType = NumberAggregatedType(`${prefix}DistinctCountAggregate`, fields, graphql_1.Int, groupedRelationFields);
        const SumType = NumberAggregatedType(`${prefix}SumAggregate`, numberFields, graphql_1.Float, groupedRelationFields);
        const AvgType = NumberAggregatedType(`${prefix}AvgAggregate`, numberFields, graphql_1.Float, groupedRelationFields);
        const MinType = AggregatedType(`${prefix}MinAggregate`, minMaxFields, groupedRelationFields);
        const MaxType = AggregatedType(`${prefix}MaxAggregate`, minMaxFields, groupedRelationFields);
        let AggResponse = class AggResponse {
        };
        tslib_1.__decorate([
            (0, graphql_1.Field)(() => GroupType, { nullable: true }),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "groupBy", void 0);
        tslib_1.__decorate([
            (0, graphql_1.Field)(() => CountType, { nullable: true }),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "count", void 0);
        tslib_1.__decorate([
            (0, graphql_1.Field)(() => DistinctCountType, { nullable: true }),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "distinctCount", void 0);
        tslib_1.__decorate([
            (0, decorators_1.SkipIf)(() => numberFields.length === 0, (0, graphql_1.Field)(() => SumType, { nullable: true })),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "sum", void 0);
        tslib_1.__decorate([
            (0, decorators_1.SkipIf)(() => numberFields.length === 0, (0, graphql_1.Field)(() => AvgType, { nullable: true })),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "avg", void 0);
        tslib_1.__decorate([
            (0, decorators_1.SkipIf)(() => minMaxFields.length === 0, (0, graphql_1.Field)(() => MinType, { nullable: true })),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "min", void 0);
        tslib_1.__decorate([
            (0, decorators_1.SkipIf)(() => minMaxFields.length === 0, (0, graphql_1.Field)(() => MaxType, { nullable: true })),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "max", void 0);
        AggResponse = tslib_1.__decorate([
            (0, graphql_1.ObjectType)(aggName)
        ], AggResponse);
        return AggResponse;
    });
}
exports.AggregateResponseType = AggregateResponseType;
//# sourceMappingURL=aggregate-response.type.js.map